package xbus2xbus //nolint:testpackage

import (
	"os"
	"path"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
	xbus "xbus.io/go-xbus/v4"
	"xbus.io/go-xbus/v4/api"
	"xbus.io/go-xbus/v4/fullenv"
)

func TestConsumer(t *testing.T) { //nolint:funlen,cyclop
	env := fullenv.NewFullenvForTest(t, "xbus2xbus", "xbus2xbus-test.yaml")
	if err := env.Start(); err != nil {
		t.Fatal(err)
	}

	defer func() {
		assert.NoError(t, env.Stop())
	}()

	c1path, err := env.ClientConfig("c1")
	if err != nil {
		t.Fatal(err)
	}
	if err := os.Chdir(path.Dir(path.Dir(c1path))); err != nil {
		t.Fatal(err)
	}
	client, err := env.LoadClient("c1")
	require.NoError(t, err)

	require.NoError(t, client.Startup(true))
	defer client.Shutdown()

	finalResult := make(chan *api.Envelope)

	finalConsumer := client.GetActorByName("final-consumer")
	if finalConsumer == nil {
		t.Fatal("No 'final-consumer'")
	}

	sub, err := finalConsumer.Subscribe(func(apc *xbus.ActorProcessingContext) error {
		reader, err := apc.ReadEnvelope("default")
		if err != nil {
			t.Error(err)
			return err
		}
		envelope, err := reader.Complete(time.Second)
		if err != nil {
			t.Error(err)
			return err
		}
		finalResult <- envelope
		return nil
	})
	if err != nil {
		t.Fatal(err)
	}

	defer func() {
		assert.NoError(t, sub.Unsubscribe())
	}()

	emitter := client.GetActorByName("first-emitter")
	if emitter == nil {
		t.Fatal("No 'first-emitter'")
	}

	if _, err := emitter.SendItems(nil, "default", "msg",
		[]api.Item{api.Item("Hello World")},
	); err != nil {
		t.Fatal(err)
	}

	select {
	case <-finalResult:
	case <-time.After(2 * time.Second):
		t.Fatal("The envelope did not reach the final consumer")
	}
}
